import pyglet
import vm_window
import random


#######################################
# VM PROC LIST
# the processor list
# this stores the process list in a tuple: (PID, processor names, CPU usage)
# Available Functionality:
#  Add (Name)
#  Remove (Name or PID)
#  Sort (Name, PID, CPU)
#  Query
#  Hide/Show Processes
#  Update
#  Render
class VMProcList:

	sort_order= ['PID','NAME','CPU']

	# INIT
	#
	def __init__(self):
		self._proc_list= []
	#enddef
	
	
	# RENDER
	# sets up what to draw
	def render(self, x, y, width, height, batch, border=2, space=2):
		textarea        = (x + border + space, y + border + space, width - (2 * (space + border)), height - (2 * (space + border)))    # x, y, width, height
		borderarea      = (x + space,          y + space,          width - (2 * space),            height - (2 * space))
		self.dim =        (x,                  y,                  width,                          height)
		
		# Text Input
		self.document = pyglet.text.document.UnformattedDocument('')
		self.document.set_style(0, len(self.document.text), {
			'font_size':10,
			'font':'Courier',
			'color':(0,255,0, 255)
		})

		self.layout = pyglet.text.layout.IncrementalTextLayout(self.document, textarea[2], textarea[3], multiline=True, batch=batch)
		self.layout.x = textarea[0]
		self.layout.y = textarea[1]
		
		# Outline / Border / TextArea Background
		self.outline_rect = vm_window.Rectangle(self.dim[0], self.dim[1], self.dim[2], self.dim[3], color=[0, 0, 0, 255], batch=batch)
		self.border_rect = vm_window.Rectangle(borderarea[0], borderarea[1], borderarea[2], borderarea[3], color=[0, 255, 0, 255], batch=batch)	
		self.textarea_rect = vm_window.Rectangle(textarea[0], textarea[1], textarea[2], textarea[3], color=[0,0,0,255], batch=batch)
		
		self.line_width= textarea[2]

		self.last_text_position= (0,0)
		self.line_text_position= (0,0)
	#enddef					   


	# SEND TEXT
	#
	def send_text(self, instr):
		self.document.insert_text(len(self.document.text), instr)
	#enddef


	# HIT TEST
	#
	def hit_test(self, x, y):
		return (0 < x - self.layout.x < self.layout.width and
				0 < y - self.layout.y < self.layout.height)
	#enddef
	
	
	##################################################
	# META FUNCTIONALITY
	##################################################
	
	# ADD PROCESS
	# returns the PID
	def add_process(self, pname, cpu_usage, pid=-1):
		new_pid= pid

		if (new_pid == -1):
			while (new_pid == -1 or (len(self._proc_list) > 0 and new_pid in self._proc_list[0])):
				new_pid= random.randrange(2000,9999)
		else:
			if (len(self._proc_list) > 0 and new_pid in self._proc_list[0]):
				return None
			#endif
		#endif
		
		new_proc= (new_pid, pname, cpu_usage)
		
		self._proc_list.append(new_proc)
		self._resort()
		return new_pid
	#enddef
	
	
	# GET PROCESS
	# get a tuple (pid, name, cpu) based on criteria. None if nothing matches criteria
	def get_process(self, pid=-1, pname=""):
		if (pid != -1):
			return filter((lambda x:(x[0] == pid)), self._proc_list)
		elif (pname != ""):
			return filter((lambda x:(x[1] == pname)), self._proc_list)	
		return None
	#enddef
	
	
	# REMOVE PROCESS
	# 
	def remove_process(self, pid=-1, pname=""):
		if (pid != -1):
			self._proc_list= filter((lambda x:(x[0] != pid)), self._proc_list)
		elif (pname != ""):
			self._proc_list= filter((lambda x:(x[1] != pname)), self._proc_list)	
		self._resort()
	#enddef
	

	# CHANGE SORT DISPLAY
	# sorts the display.  Can sort on "PID", "NAME" or "CPU"
	def change_sort_display(self, sorton):
		if (sorton in VMProcList.sort_order):
			VMProcList.sort_order.remove(sorton)
			VMProcList.sort_order.insert(0, sorton)
		self._resort()
	#enddef
	
	
	# WRITE TEXT
	#
	def write_text(self):
		newtext= "PID    NAME    CPU\n"
		for proc in self._proc_list:
			newtext= newtext + str(proc) + "\n"
		#endfor
		self.document.text= newtext;
	#enddef
	
	
	# RESORT / SORT_CMP
	# will sort the list and then change the display
	def _resort(self):
		self._proc_list.sort(self._sort_cmp)
		self.write_text()
	#enddef
	def _sort_cmp(self,a,b):
		for sortby in VMProcList.sort_order:
			if (sortby == "PID" and a[0] != b[0]):
				return (a[0] - b[0])
			elif (sortby == "CPU" and a[2] != b[2]):
				return (a[2] - b[2])
			elif (sortby == "NAME"):
				av= a[1].upper()
				bv= b[1].upper()
				if (av > bv):
					return 1
				elif (av < bv):
					return -1
			#endif
		#endfor
	#enddef
#endclass





#######################################
# VM PROMPT
# the command line
class VMPrompt:
	# INIT
	#
	def __init__(self):
		pass	
	#enddef
	
	
	# RENDER
	# for placement, adds itself to the VMWindow Batch for drawing
	def render(self, x, y, width, batch, border=2, space=2):
		textarea        = (x + border + space, y + border + space, width - (2 * (space + border)), -1)    # x, y, width, height
		borderarea      = (x + space,          y + space,          width - (2 * space),            -1)
		self.dim =        (x,                  y,                  width,                          -1)
		
		# Prompt Indicator
		prompt_indicator= pyglet.text.Label('>', color=(0, 255, 0, 255), x=textarea[0]+2, y=textarea[1]+2, batch=batch)
		
		# Text Input
		self.document = pyglet.text.document.UnformattedDocument('')
		self.document.set_style(0, len(self.document.text), {
			'font_size':10,
			'font':'Courier',
			'color':(0,255,0, 255)
		})
		self.font = self.document.get_font()
		height= (self.font.ascent - self.font.descent)
		

		textarea        = (x + border + space, y + border + space, width - (2 * (space + border)), height)    # x, y, width, height
		borderarea      = (x + space,          y + space,          width - (2 * space),            height + (2 * border))
		self.dim = (x,                  y,                  width,                          height + (2 * (space + border)))

		self.layout = pyglet.text.layout.IncrementalTextLayout(self.document, width=textarea[2], height=textarea[3], multiline=False, batch=batch)
		self.layout.x = textarea[0] + 12  # beyond '>'
		self.layout.y = textarea[1]
		
		# Caret
		self.caret = pyglet.text.caret.Caret(self.layout)
		self.caret.color = [0,255,0]

		# Outline / Border / TextArea Background
		self.outline_rect = vm_window.Rectangle(self.dim[0], self.dim[1], self.dim[2], self.dim[3], color=[0, 0, 0, 255], batch=batch)
		self.border_rect = vm_window.Rectangle(borderarea[0], borderarea[1], borderarea[2], borderarea[3], color=[0, 255, 0, 255], batch=batch)	
		self.textarea_rect = vm_window.Rectangle(textarea[0], textarea[1], textarea[2], textarea[3], color=[0,0,0,255], batch=batch)		
	#enddef							   


	# HIT TEST
	#
	def hit_test(self, x, y):
		return (0 < x - self.layout.x < self.layout.width and
				0 < y - self.layout.y < self.layout.height)
	#enddef
	
	
	# GET TEXT
	#
	def get_text(self):
		ret= self.document.text
		self.document.text= ""

		return ret
	#enddef
#endclass



#######################################
# VM CONSOLE
# output box
class VMConsole:
	
	scroll_sensititvity= 15.0
	
	# INIT
	#
	def __init__(self):
		pass
	#enddef
	
	
	# RENDER
	# sets up what to draw
	def render(self, x, y, width, height, batch, border=2, space=2):
		textarea        = (x + border + space, y + border + space, width - (2 * (space + border)), height - (2 * (space + border)))    # x, y, width, height
		borderarea      = (x + space,          y + space,          width - (2 * space),            height - (2 * space))
		self.dim = (x,                  y,                  width,                          height)
		
		# Text Input
		self.document = pyglet.text.document.UnformattedDocument('')
		self.document.set_style(0, len(self.document.text), {
			'font_size':10,
			'font':'Courier',
			'color':(0,255,0, 255)
		})

		self.layout = pyglet.text.layout.IncrementalTextLayout(self.document, textarea[2], textarea[3], multiline=True, batch=batch)
		self.layout.x = textarea[0]
		self.layout.y = textarea[1]
		
		# Outline / Border / TextArea Background
		self.outline_rect = vm_window.Rectangle(self.dim[0], self.dim[1], self.dim[2], self.dim[3], color=[0, 0, 0, 255], batch=batch)
		self.border_rect = vm_window.Rectangle(borderarea[0], borderarea[1], borderarea[2], borderarea[3], color=[0, 255, 0, 255], batch=batch)	
		self.textarea_rect = vm_window.Rectangle(textarea[0], textarea[1], textarea[2], textarea[3], color=[0,0,0,255], batch=batch)
		
		self.line_width= textarea[2]

		self.last_text_position= (0,0)
		self.line_text_position= (0,0)
	#enddef


	# SEND TEXT
	#
	def send_text(self, instr):
		instr.replace("\r","") 
		instr.replace("\n","")
		instr+= "\n"
		self.document.insert_text(len(self.document.text), instr)
		self.layout.ensure_line_visible(self.layout.get_line_count()-1)
	#enddef


	# HIT TEST
	#
	def hit_test(self, x, y):
		return (0 < x - self.layout.x < self.layout.width and
				0 < y - self.layout.y < self.layout.height)
	#enddef
	
	
	# SCROLL
	def scroll(self, y):
		self.layout.view_y += VMConsole.scroll_sensititvity * y
	#enddef
	
	
	# ADD TEXT
	#
	def add_text(self, text):
		# need to break text into individual letters
		space_width= self.font.get_glyphs(" ")[0].width
		for letter in text:
			if (letter.isspace()):
				self.last_text_position= (self.caret.position, self.line_text_position[1])
			#endif

			glyph= self.font.get_glyphs(letter)
			self.line_text_position= (self.caret.position, self.line_text_position[1] + glyph[0].width)
			#print (letter, glyph[0].width, self.line_text_position, self.last_text_position,self.line_width)

			if (self.line_text_position[1] >= self.line_width):
				if (self.last_text_position == (0,0)):
					self.document.insert_text(self.line_text_position[0], "\n")
					self.caret.position+= 1

					self.line_text_position= (self.caret.position, glyph[0].width)
				else:
					savepos= self.caret.position
					#remove space
					self.document.delete_text(self.last_text_position[0], self.last_text_position[0] + 1)
					self.line_text_position= (self.caret.position, self.line_text_position[1] - space_width)

					#add newline
					self.document.insert_text(self.last_text_position[0], "\n")
					self.line_text_position= (savepos, self.line_text_position[1] - self.last_text_position[1])

					self.caret.position= savepos
					self.last_text_position= (0,0)

				#endif
			#endif

			self.caret.on_text(letter)

		#endfor
	#enddef
#endclass
